from latticegen.latticegeneration import *
from latticegen.singularities import hexlattice_gen_singularity, singularity_shift 

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions



"""Code for generating (quasi-)lattices and the corresponding k-vectors"""

import numpy as np
import dask.array as da
import itertools as itert

from latticegen.transformations import rotate, rotation_matrix, scaling_matrix, apply_transformation_matrix, a_0_to_r_k


def generate_ks(r_k, theta, kappa=1., psi=0., sym=6):
    """Generate k-vectors from given parameters.

    Parameters
    ----------
    r_k : float
        length of lattice vectors in k-space. Larger `r_k` correspond
        to smaller real space lattice constants.
    theta : float
        Angle of the first lattice vector with respect to positive
        horizontal.
    kappa : float, default: 1
        strain/deformation magnitude. 1 corresponds to no strain.
    psi : float, default: 0
        Principal strain direction with respect to horizontal
        in degrees.
    sym : int, default 6
        Rotational symmetry of the unstrained lattice.
    Returns
    -------
    ks : np.array (2x(`sym` + 1))
    """
    W = rotation_matrix(np.deg2rad(theta))
    V = rotation_matrix(np.deg2rad(psi))
    D = scaling_matrix(kappa)
    ks = np.stack([rotate(np.array([r_k, 0]), 2*np.pi/sym*i) for i in range(sym)]
                  + [(0, 0)])
    ks = apply_transformation_matrix(ks,  V.T @ D @ V @ W)
    return ks


def combine_ks(kvecs, order=1, return_counts=False):
    """Generate all possible different sums of kvecs upto order.

    Parameters
    ----------
    kvecs : array-like 2xN
        k vectors to combine.
    order: int, default: 1
        Number of different `kvecs` to combine for each resulting `tks`.
    return_counts : bool, default False
        if True, also return the number of possible combinations
        for each different combination.

    Returns
    -------
    tks : (2xM) array of float
        All possible unique combinations of `kvecs` upto `order`
    counts : (1xM) array of int, optional
        Number of combinations for each vectors in tks.
    """
    # Yes, this is probably not the smartest way to do this
    tks = np.array([np.sum(k_prod, axis=0) for k_prod in itert.product(*[kvecs]*order)])
    return np.unique(tks, return_counts=return_counts, axis=0)


def hexlattice_gen_fast(r_k, theta, order, size=500,
                        kappa=1., psi=0., shift=np.array((0, 0))):
    """Generate a regular hexagonal lattice.

    Speed optimized version, losing some mathematical precision.
    Tested to be accurate down to max(1e-10*r_k, 1e-10) w.r.t. the regular function.
    The lattice is generated from the six 60 degree rotated k-vectors
    of length r_k, further rotated by `theta` degrees.
    Size either an int for a size*size lattice, or tuple (N,M) for a rectangle N*M.
    Optionally, the lattice can be strained by a factor kappa in direction psi[1].

    With higher order frequency components upto order `order`

    The generated lattice gets returned as a dask array.

    See Also
    --------
    anylattice_gen
    """
    if not isinstance(size, tuple):
        size = (size, size)
    xx = da.arange(-1*size[0]/2, size[0]/2)[:, None]
    yy = da.arange(-1*size[1]/2, size[1]/2)[None]
    ks = generate_ks(r_k, theta, kappa, psi, sym=6)
    rks, k_c = combine_ks(ks, order=order, return_counts=True)
    rks = da.from_array(rks, chunks=(32, 2))
    phases = (xx + shift[0])[..., None]*rks[:, 0], (yy + shift[1])[..., None]*rks[:, 1]
    iterated = k_c * (np.exp(np.pi*2*1j * phases[0]) * np.exp(np.pi*2*1j * phases[1]))
    iterated = iterated.real.sum(axis=-1)
    # Now add the second shifted sublattice lattice to get a hexagonal lattice
    x = np.array([ks[1], -ks[2]])
    shift2 = (shift.T + (np.linalg.inv(x).T/3).sum(axis=0)).T  # Don't ask, this works
    phases2 = (xx + shift2[0])[..., None]*rks[:, 0], (yy + shift2[1])[..., None]*rks[:, 1]
    iterated += (k_c*np.exp(np.pi*2*1j * phases2[0])*np.exp(np.pi*2*1j * phases2[1])).real.sum(axis=-1)
    return iterated


def hexlattice_gen(r_k, theta, order, size=500,
                   kappa=1., psi=0., shift=np.array((0, 0)), **kwargs):
    """Generate a regular hexagonal lattice.
    The lattice is generated from the six 60 degree rotated k-vectors
    of length r_k, further rotated by `theta` degrees.
    Optionally, the lattice can be strained by a factor kappa in direction psi [1].
    Rendered with higher order frequency components upto order `order`.

    Parameters
    ----------
    r_k : float
        length of lattice vectors in k-space. Larger `r_k` correspond
        to smaller real space lattice constants.
    theta : float
        Angle of the first lattice vector with respect to positive
        horizontal.
    order : int
        Order upto which to generate higher frequency components
        by combining lattice vectors
    size: int, or pair of int, default: 500
        Size of the resulting lattice in pixels. if int, the
        returned lattice will be square.
    kappa : float, default: 1
        strain/deformation magnitude. 1 corresponds to no strain.
    psi : float, default: 0
        Principal strain direction with respect to horizontal.
    shift : iterable or array, optional
        shift of the lattice. Either a pair (x,y) global shift,
        or an (2xNxM) array where (NxM) corresponds to `size`.
    **kwargs : dict
        Keyword arguments to be passed to `anylattice_gen`

    Returns
    -------
    lattice : Dask array
        The generated lattice.

    See Also
    --------
    anylattice_gen
    """
    sublattice_a = anylattice_gen(r_k, theta, order, symmetry=6, size=size,
                                  kappa=kappa, psi=psi, shift=shift, **kwargs)
    # Now add the second shifted sublattice lattice to get a hexagonal lattice
    ks = generate_ks(r_k, theta, kappa, psi, sym=6)
    x = np.array([ks[1], -ks[2]])
    if r_k < 1e-10:
        shift2 = (shift.T + (np.linalg.inv(x).T/3).sum(axis=0)).T  # Don't ask, this works
    else:
        shift2 = (shift.T + (np.linalg.inv(x / r_k).T/(3*r_k)).sum(axis=0)).T
    sublattice_b = anylattice_gen(r_k, theta, order, symmetry=6, size=size,
                                  kappa=kappa, psi=psi, shift=shift2, **kwargs)
    return sublattice_a + sublattice_b


def squarelattice_gen(r_k, theta, order, size=500,
                      kappa=1., psi=0., shift=np.array((0, 0)), **kwargs):
    """Generate a regular square lattice.

    The lattice is generated from the four 90 degree rotated k-vectors
    of length `r_k`, further rotated by `theta` degrees.
    Optionally, the lattice can be strained by a factor `kappa` in direction `psi`[1].
    Rendered with higher order frequency components upto order `order`.

    Parameters
    ----------
    r_k : float
        length of lattice vectors in k-space. Larger `r_k` correspond
        to smaller real space lattice constants.
    theta : float
        Angle of the first lattice vector with respect to positive
        horizontal.
    order : int
        Order upto which to generate higher frequency components
        by combining lattice vectors
    size: int, or pair of int, default: 500
        Size of the resulting lattice in pixels. if int, the
        returned lattice will be square.
    kappa : float, default: 1
        strain/deformation magnitude. 1 corresponds to no strain.
    psi : float, default: 0
        Principal strain direction with respect to horizontal.
    shift : iterable or array, optional
        shift of the lattice. Either a pair (x,y) global shift,
        or an (2xNxM) array where (NxM) corresponds to `size`.
    **kwargs : dict
        Keyword arguments to be passed to `anylattice_gen`

    Returns
    -------
    lattice : Dask array
        The generated lattice.

    See Also
    --------
    anylattice_gen
    """
    return anylattice_gen(r_k, theta, order,
                          symmetry=4, size=size,
                          kappa=kappa, psi=psi,
                          shift=shift, **kwargs)


def trilattice_gen(r_k, theta, order, size=500,
                   kappa=1., psi=0., shift=np.array((0, 0)), **kwargs):
    """Generate a regular trigonal lattice.

    The lattice is generated from the six 60 degree rotated k-vectors
    of length `r_k`, further rotated by `theta` degrees.
    Optionally, the lattice can be strained by a factor `kappa` in direction `psi`[1].
    Rendered with higher order frequency components upto order `order`.

    Parameters
    ----------
    r_k : float
        length of lattice vectors in k-space. Larger `r_k` correspond
        to smaller real space lattice constants.
    theta : float
        Angle of the first lattice vector with respect to positive
        horizontal.
    order : int
        Order upto which to generate higher frequency components
        by combining lattice vectors
    size: int, or pair of int, default: 500
        Size of the resulting lattice in pixels. if int, the
        returned lattice will be square.
    kappa : float, default: 1
        strain/deformation magnitude. 1 corresponds to no strain.
    psi : float, default: 0
        Principal strain direction with respect to horizontal.
    shift : iterable or array, optional
        shift of the lattice. Either a pair (x,y) global shift,
        or an (2xNxM) array where (NxM) corresponds to `size`.
    **kwargs : dict
        Keyword arguments to be passed to `anylattice_gen`

    Returns
    -------
    lattice : Dask array
        The generated lattice.

    See Also
    --------
    anylattice_gen, hexlattice_gen
    """
    return anylattice_gen(r_k, theta, order,
                          symmetry=6, size=size,
                          kappa=kappa, psi=psi,
                          shift=shift, **kwargs)


def anylattice_gen(r_k, theta, order, symmetry=6, size=500,
                   kappa=1., psi=0., shift=np.array((0, 0)),
                   chunks=(-1, -1)):
    """Generate a regular lattice of any symmetry.
    The lattice is generated from the `symmetry` `360/symmetry` degree rotated k-vectors
    of length `r_k`, further rotated by `theta` degrees.
    Size either an int for a size*size lattice, or tuple (N,M) for a rectangle N*M.
    Optionally, the lattice can be strained by a factor kappa in direction psi[1].

    With higher order frequency components upto order `order`

    Parameters
    ----------
    r_k : float
        length of lattice vectors in k-space. Larger `r_k` correspond
        to smaller real space lattice constants.
    theta : float
        Angle of the first lattice vector with respect to positive
        horizontal.
    order : int
        Order upto which to generate higher frequency components
        by combining lattice vectors
    symmetry : int
        symmetry of the lattice.
    size: int, or pair of int, default: 500
        Size of the resulting lattice in pixels. if int, the
        returned lattice will be square.
    kappa : float, default: 1
        strain/deformation magnitude. 1 corresponds to no strain.
    psi : float, default: 0
        Principal strain direction with respect to horizontal.
    shift : iterable or array, optional
        shift of the lattice. Either a pair (x,y) global shift,
        or an (2xNxM) array where (NxM) corresponds to `size`.
    chunks : int or pair of int, optional
        dask chunks in which to divide the returned `lattice`.

    Returns
    -------
    lattice : Dask array
        The generated lattice.

    See Also
    --------
    generate_ks

    References
    ----------
    [1] T. Benschop et al., 2020, https://doi.org/10.1103/PhysRevResearch.3.013153
    """
    if not isinstance(size, tuple):
        size = (size, size)
    if not isinstance(chunks, tuple):
        chunks = (chunks, chunks)
    xx = da.arange(-1*size[0]/2, size[0]/2, chunks=chunks[0])[:, None]
    yy = da.arange(-1*size[1]/2, size[1]/2, chunks=chunks[1])[None]
    ks = generate_ks(r_k, theta, kappa, psi, sym=symmetry)
    rks, k_c = combine_ks(ks, order=order, return_counts=True)
    rks = da.from_array(rks, chunks=(13, 2))
    phases = (xx + shift[0])*rks[:, 0, None, None] + (yy + shift[1])*rks[:, 1, None, None]
    iterated = k_c[:, None, None]*np.exp(np.pi*2*1j * phases)
    iterated = iterated.sum(axis=0)
    return iterated.real


def anylattice_gen_np(r_k, theta, order=1, symmetry=6, size=50,
                      kappa=1., psi=0., shift=np.array((0, 0))):
    """Generate a regular lattice of any symmetry in pure numpy.

    The lattice is generated from the `symmetry` `360/symmetry` degree rotated k-vectors
    of length r_k, further rotated by `theta` degrees.
    Size either an int for a size*size lattice, or tuple (N,M) for a rectangle N*M.
    Optionally, the lattice can be strained by a factor kappa in direction psi[1].

    With higher order frequency components upto order `order`

    The generated lattice gets returned as a numpy array.
    Only usable for small values of size and order.

    See Also
    --------
    anylattice_gen
    """
    if not isinstance(size, tuple):
        size = (size, size)
    xx = np.arange(-1*size[0]/2, size[0]/2)[:, None]
    yy = np.arange(-1*size[1]/2, size[1]/2)[None]
    ks = generate_ks(r_k, theta, kappa, psi, sym=symmetry)
    rks, k_c = combine_ks(ks, order=order, return_counts=True)
    if not np.isfinite(2*np.pi * r_k * (max(size) + max(shift)) / 2):
        print("Warning, using more than float-max periods in a single lattice.")
    phases = (xx + shift[0])*rks[:, 0, None, None] + (yy + shift[1])*rks[:, 1, None, None]
    iterated = k_c[:, None, None]*np.exp(np.pi*2*1j * phases)
    iterated = iterated.sum(axis=0)
    return iterated.real


def physical_lattice_gen(a_0, theta, order, pixelspernm=10, symmetry='hexagonal',
                         size=500, **kwargs):
    """Generate a physical lattice

    Wraps anylattice_gen.
    Using a lattice constant `a_0` in nm and a resolution in pixels per nm,
    generate a rendering of a lattice of `size` pixels.
    Optionally, the lattice can be strained by a factor kappa in direction psi[1].

    With higher order frequency components upto order `order`

    Parameters
    ----------
    a_0 : float
        lattice constant in nm
    theta : float
        Angle of the first lattice vector with respect to positive
        horizontal.
    order : int
        Order upto which to generate higher frequency components
        by combining lattice vectors
    pixelspernm : float, default: 10
        number of pixels per nanometer.
    symmetry : {'hexagonal', 'trigonal', 'square'}
        symmetry of the lattice.
    size: int, or pair of int, default: 500
        Size of the resulting lattice in pixels. if int, the
        returned lattice will be square.
    **kwargs : dict
        Keyword arguments to be passed to `anylattice_gen`

    Returns
    -------
    lattice : Dask array
        The generated lattice.

    See Also
    --------
    anylattice_gen

    """
    r_k = 1 / (np.sin(2*np.pi / symmetry) * pixelspernm * a_0)
    if symmetry == 'square':
        r_k = a_0_to_r_k(a_0 * pixelspernm, 4)
        return anylattice_gen(r_k, theta, order,
                              symmetry=4, **kwargs)
    else:
        r_k = a_0_to_r_k(a_0 * pixelspernm, 6)
        if symmetry == 'hexagonal':
            return hexlattice_gen(r_k, theta, order,
                                  **kwargs)
        elif symmetry == 'trigonal':
            return anylattice_gen(r_k, theta, order,
                                  symmetry=6, **kwargs)
        else:
            raise Exception("symmetry {} is unknown".format(symmetry))
    return anylattice_gen(r_k, theta, order,
                          symmetry=6, **kwargs)



"""Code to generate edge dislocations/singularities in lattices"""
import numpy as np
import dask.array as da
import itertools as itert

from latticegen.transformations import (
    rotate,
    rotation_matrix,
    apply_transformation_matrix,
    wrapToPi,
)
from latticegen.latticegeneration import hexlattice_gen


def hexlattice_gen_singularity(r_k, theta, order, size=250,
                               position=[0, 0], shift=np.array([0, 0]),
                               **kwargs):
    """Generate a hexagonal lattice with a singularity.

    Singularity is shifted `position` from the center.
    Not yet equivalent to hexlattice_gen_singularity_legacy.

    Parameters
    ----------
    r_k : float
        length of lattice vectors in k-space. Larger `r_k` correspond
        to smaller real space lattice constants.
    theta : float
        Angle of the first lattice vector with respect to positive
        horizontal.
    order : int
        Order upto which to generate higher frequency components
        by combining lattice vectors
    size: int, or pair of int, default: 500
        Size of the resulting lattice in pixels. if int, the
        returned lattice will be square.
    kappa : float, default: 1
        strain/deformation magnitude. 1 corresponds to no strain.
    position : iterable, default [0, 0]
        [x, y] position of the singularity in pixels with
        respect to the center.
    shift : iterable or array, optional
        shift of the lattice. Either a pair (x,y) global shift,
        or an (2xNxM) array where (NxM) corresponds to `size`.
    **kwargs : dict
        Keyword arguments to be passed to `hexlattice_gen`

    Returns
    -------
    lattice : Dask array
        The generated lattice with singularity

    See Also
    --------
    hexlattice_gen, singularity_shift
    """
    shift2 = shift + singularity_shift(r_k, theta, size, position)
    return hexlattice_gen(r_k, theta, order, size, shift=shift2, **kwargs)


def singularity_shift(r_k, theta, size=250, position=[0, 0],
                      alpha=0., symmetry=6):
    """Generate the shift of an edge dislocation.

    Generate the shift / displacement field
    for a hexagonal lattice of `size`, with the Burgers vector
    at angle `alpha` (radians) w.r.t. `theta` (in degrees).
    Burgers vector has length `np.sin(2*np.pi/symmetry) / r_k`,
    to create a first order edge dislocation.

    Parameters
    ----------
    r_k : float
        length of lattice vectors in k-space. Larger `r_k` correspond
        to smaller real space lattice constants.
    theta : float
        Angle of the first lattice vector with respect to positive
        horizontal.
    order : int
        Order upto which to generate higher frequency components
        by combining lattice vectors
    size: int, or pair of int, default: 500
        Size of the resulting lattice in pixels. if int, the
        returned lattice will be square.
    position : iterable, default: [0, 0]
        [x, y] position of the singularity in pixels with
        respect to the center.
    alpha : float, default 0.0
        angle in radians of the Burgers vector with respect
        to the first lattice vector.
    symmetry : int, default: 6
        symmetry of the lattice.

    Returns
    -------
    shift : ndarray
        The generated shift corresponding to a singularity,
        which can be passed to anylattice_gen

    See Also
    --------
    anylattice_gen
    """
    if not isinstance(size, tuple):
        size = (size, size)
    xx = da.arange(-size[0]/2, size[0]/2)[:, None] - position[0]
    yy = da.arange(-size[1]/2, size[1]/2)[None] - position[1]
    phi = np.arctan2(yy, xx)
    phiprime = phi / (2*np.pi)
    a_0 = 1 / np.sin(2*np.pi / symmetry) / r_k
    xp = a_0*np.sin(alpha - np.deg2rad(theta)) * phiprime
    yp = a_0*np.cos(alpha - np.deg2rad(theta)) * phiprime
    shift = np.array([xp, yp])
    return shift


def hexlattice_gen_singularity_legacy(r_k, theta, order, size=250):
    """Generate a regular hexagonal lattice of `2*size` times `2*size`.
    The lattice is generated from the six 60 degree rotations of `k0`,
    further rotated by `theta` degrees. With higher order frequency
    components upto order `order` and containing a singularity in the center.
    `theta != 0` does not yield a correct lattice.

    The generated lattice gets returned as a dask array.
    """
    if not isinstance(size, tuple):
        size = (size, size)
    xx = da.arange(-size[0]/2, size[0]/2)[:, None]
    yy = da.arange(-size[1]/2, size[1]/2)[None]
    ks = np.stack(
        [rotate(np.array([r_k, 0]), np.pi / 3 * i) for i in range(6)] + [(0, 0)]
    )
    W = rotation_matrix(np.deg2rad(theta))
    ks = apply_transformation_matrix(ks, W)
    # Yes, this is probably not the smartest way to do this
    tks = np.array([np.sum(ksp, axis=0) for ksp in itert.product(*[ks] * order)])
    rks, k_c = np.unique(tks, return_counts=True, axis=0)
    rks = da.from_array(rks, chunks=(13, 2))
    x = np.array([ks[1], -ks[2]])
    shift = (np.linalg.inv(x).T / 3).sum(axis=0)  # Don't ask, this works
    xp = xx + wrapToPi(-np.deg2rad(120 + theta) + np.arctan2(yy, xx)) / 2 / np.pi / r_k
    yp = (yy + 1/np.sqrt(3) * (wrapToPi(-np.deg2rad(120 + theta) + np.arctan2(yy, xx)))
          / 2/np.pi/r_k)
    iterated = k_c[:, None, None] * np.exp(np.pi*2j
               * ((xp - 1 * shift[0]) * rks[:, 0, None, None]
                  + (yp - 1 * shift[1]) * rks[:, 1, None, None]))
    iterated = iterated.sum(axis=0)
    # Now add the second shifted sublattice lattice to get a hexagonal lattice
    iterated += (k_c[:, None, None] * np.exp(np.pi * 2j
                 * ((xp + shift[0]) * rks[:, 0, None, None]
                    + (yp + shift[1]) * rks[:, 1, None, None]))
                 ).sum(axis=0)
    return iterated.real



import setuptools
import versioneer

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

with open('requirements_test.txt') as f:
    requirements_test = f.read().splitlines()

setuptools.setup(
    name="latticegen",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    author="T.A. de Jong",
    author_email="tobiasadejong@gmail.com",
    description="A small package to create images of atomic lattices",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/TAdejong/moire-lattice-generator",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering",
    ],
    python_requires='>=3.7',
    install_requires=requirements,
    test_requires=requirements_test,
    test_suite="pytest",
)



"""Common code for geometrical transformations"""
import numpy as np


def rotation_matrix(angle):
    """Create a rotation matrix

    an array corresponding to the 2D transformation matrix
    of a rotation over `angle`.

    Parameters
    ----------
    angle : float
        rotation angle in radians

    Returns
    -------
    ndarray (2x2)
        2D transformation matrix corresponding
        to the rotation
    """
    return np.array([[np.cos(angle), -np.sin(angle)],
                     [np.sin(angle), np.cos(angle)]])


def scaling_matrix(kappa, dims=2):
    """Create a scaling matrix

    Creates a numpy array containing the `dims`-dimensional
    scaling matrix scaling the first dimension by `kappa`.

    Parameters
    ----------
    kappa : float
        scaling factor of first dimension
    dims : int, default: 2
        number of dimensions
    Returns
    -------
    ndarray (dims x dims)
        scaling matrix corresponding
        to scaling the first dimension by a factor `kappa`
    """
    return np.diag([kappa]+[1]*(dims-1))


def strain_matrix(epsilon, delta=0.16, axis=0):
    """Create a scaling matrix corresponding to uniaxial strain

    Only works for the 2D case.

    Parameters
    ----------
    epsilon : float
        applied strain
    delta : float, default 0.16
        Poisson ratio. default value corresponds to graphene
    axis : {0, 1}
        Axis along which to apply the strain.

    Returns
    -------
    ndarray (2 x 2)
        scaling matrix corresponding
        to `epsilon` strain along `axis`
    """
    scaling = np.full(2, 1 - delta*epsilon)
    scaling[axis] = 1 + epsilon
    return np.diag(scaling)


def apply_transformation_matrix(vecs, matrix):
    """Apply transformation matrix to a list of vectors.

    Apply transformation matrix `matrix` to a list of vectors `vecs`.
    `vecs` can either be a list of vectors, or a NxM array, where N is
    the number of M-dimensional vectors.

    Parameters
    ----------
    vecs : 2D array or iterable
        list of vectors to be transformed
    matrix : 2D array
        Array corresponding to the transformation matrix

    Returns
    -------
    2D array
        Transformed vectors
    """
    avecs = np.asanyarray(vecs)
    return avecs @ matrix.T


def rotate(vecs, angle):
    "Rotate 2D vectors `vecs` by `angle` radians around the origin."
    return apply_transformation_matrix(vecs, rotation_matrix(angle))


def wrapToPi(x):
    """Wrap all values of `x` to the interval [-pi,pi)"""
    r = (x + np.pi) % (2*np.pi) - np.pi
    return r


def a_0_to_r_k(a_0, symmetry=6):
    crossprodnorm = np.sin(2*np.pi / symmetry)
    r_k = 1 / a_0 / crossprodnorm
    return r_k


def r_k_to_a_0(r_k, symmetry=6):
    crossprodnorm = np.sin(2*np.pi / symmetry)
    a_0 = 1 / r_k / crossprodnorm
    return a_0
